<?php

/**
 * @file
 * Administrative page callbacks for the multiping module.
 */

/**
 * Menu callback: Overview multiping services.
 */
function multiping_admin_list() {
  // Table with services.
  $header = array(
    array('data' => t('Site name'), 'field' => 'name'),
    array('data' => t('Last update'), 'field' => 'lastping'),
    array('data' => t('Operations')),
  );

  $query = db_select('multiping');
  $query->fields('multiping');
  $query->extend('TableSort')->orderByHeader($header);
  $result = $query->execute();

  $destination = drupal_get_destination();
  $rows = array();
  foreach ($result as $service) {
    $row = array('data' => array());
    $row['data'][] = check_plain($service->name);
    $row['data'][] = $service->lastping ? format_date($service->lastping, 'short') : t('never');
    $operations = array();
    $operations['edit'] = array(
      'title' => t('edit'),
      'href' => "admin/config/services/multiping/{$service->id}/edit",
      'query' => $destination,
    );
    $operations['ping'] = array(
      'title' => t('ping'),
      'href' => "admin/config/services/multiping/{$service->id}/ping",
      'query' => $destination,
    );
    $operations['delete'] = array(
      'title' => t('delete'),
      'href' => "admin/config/services/multiping/{$service->id}/delete",
      'query' => $destination,
    );
    $row['data']['operations'] = array(
      'data' => array(
        '#theme' => 'links',
        '#links' => $operations,
        '#attributes' => array('class' => array('links', 'inline')),
      ),
    );
    $rows[] = $row;
  }
  $build['miltiping_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No services available. <a href="@add">Add service</a>.', array('@add' => url('admin/config/services/multiping/add'))),
  );
  return $build;
}

/**
 * Menu callback: Global settings form.
 */
function multiping_settings_form($form, &$form_state) {
  //TODO: Add validation
  $form['multiping_time_between_pings'] = array(
    '#type' => 'textfield',
    '#title' => t('Time between pings'),
    '#default_value' => variable_get('multiping_time_between_pings', 10),
    '#size' => 8,
    '#maxlength' => 4,
    '#description' => t('Minimum number of minutes between two pings to same site'),
    '#required' => TRUE,
  );
  $form['multiping_pingatonce'] = array(
    '#type' => 'checkbox',
    '#title' => t('Ping after post'),
    '#default_value' => variable_get('multiping_pingatonce', FALSE),
    '#description' => t('Ping directly after post/update (instead of pinging during the cron job runs)'),
  );
  $form['multiping_timeout'] = array(
    '#type' => 'select',
    '#title' => t('Timeout for each ping'),
    '#default_value' => variable_get('multiping-timeout', 3),
    '#options' => drupal_map_assoc(array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)),
    '#description' => t('Time to ping each service in seconds')
  );
  return system_settings_form($form);
}


/**
 * Form builder for add/edit service.
 */
function multiping_edit_service_form($form, &$form_state, $pingservice = array()) {
  //TODO: Add validation

  $defaults = array(
    'id' => NULL,
    'name' => '',
    'url' => 'http://',
    'method' => 'weblogUpdates.ping',
    'flags' => array(
      'when_active' => 1,
      'when_taxonomy' => 0,
      'when_nodetype' => 0,
      'when_frontpage' => 0,
    ),
    'submitmainrss' => 1,
    'voc' => array(),
    'nodetypes' => array(),
  );
  $pingservice += $defaults;

  $form['id'] = array('#type' => 'value', '#value' => $pingservice['id']);
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Site name'),
    '#default_value' => $pingservice['name'],
    '#maxlength' => 100,
    '#description' => t('Displayed name of the site to be pinged'),
    '#required' => TRUE,
  );
  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#default_value' => $pingservice['url'],
    '#maxlength' => 255,
    '#description' => t('URL of the ping service. ' .
      'The following replacements will be done:<br />' .
      '%name - Site name (and slogan)<br />' .
      '%url - Site base URL<br />' .
      '%rss - URL of RSS feed'),
    '#required' => TRUE,
  );
  $form['method'] = array(
    '#type' => 'textfield',
    '#title' => t('Method name'),
    '#default_value' => $pingservice['method'],
    '#maxlength' => 50,
    '#description' => t('If this is an XML service: Name of the method to be called<br/>weblogUpdates.ping or weblogUpdates.extendedPing'),
    '#required' => FALSE,
  );
  $options = array(
    '1' => t('Always ping'),
    '0' => t('Don\'t ping'),
  );
  if (module_exists('taxonomy')) {
    $options['2'] = t('Ping only for nodes with the following taxonomy');
  }
  $form['when_active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Service active'),
    '#default_value' => $pingservice['flags']['when_active'],
    '#description' => t('Activate or deactivate this service'),
  );
  if (module_exists('taxonomy')) {
    $form['when_taxonomy'] = array(
      '#type' => 'checkbox',
      '#title' => t('Ping only for selected taxonomies'),
      '#default_value' => $pingservice['flags']['when_taxonomy'],
      '#description' => t('Send a ping only if the node belongs to one of the following categories'),
    );
    // Vocabularies indexed by machine_name.
    $vocabularies = taxonomy_get_vocabularies();
    $options = array();
    foreach ($vocabularies as $vocabulary) {
      $options[$vocabulary->vid] = $vocabulary->name;
    }
    $form['voc'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Taxonomy terms when to ping'),
      '#default_value' => $pingservice['voc'],
      '#options' => $options,
      '#description' => t('Ping this service if any of the taxonomy terms selected above matches the respective node'),
      '#states' => array(
        'visible' => array(
          ':input[name="when_taxonomy"]' => array('checked' => TRUE),
        ),
      ),
    );
    // TODO upgrade to taxonomy fields.
    $form['voc'] = array('#type' => 'value', '#value' => $pingservice['voc']);
    $form['submitmainrss'] = array(
      '#type' => 'checkbox',
      '#title' => t('Send main RSS feed in ping'),
      '#default_value' => $pingservice['submitmainrss'],
      '#description' => t('If checked, the main RSS feed is sent; otherwise, an RSS feed containing only articles from the selected taxonomies is used.'),
    );
  }

  $form['when_nodetype'] = array(
    '#type' => 'checkbox',
    '#title' => t('Ping only for selected node types'),
    '#default_value' => $pingservice['flags']['when_nodetype'],
    '#description' => t('Send a ping only if the node is one of the following node types'),
  );
  $form['nodetypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Node types when to ping'),
    '#default_value' => $pingservice['nodetypes'],
    '#options' => node_type_get_names(),
    '#description' => t('Ping this service if any of the node types selected above matches the respective node'),
    '#states' => array(
      'visible' => array(
        ':input[name="when_nodetype"]' => array('checked' => TRUE),
      ),
    ),
  );
  $form['when_frontpage'] = array(
    '#type' => 'checkbox',
    '#title' => t('Ping only for nodes promoted to front page'),
    '#default_value' => $pingservice['flags']['when_frontpage'],
    '#description' => t('Send a ping only if the node is promoted to the front page'),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Submit handler for multiping_edit_service_form().
 */
function multiping_edit_service_form_submit($form, &$form_state) {
  $service = $form_state['values'];
  $service['voc'] = array_filter($service['voc']);
  $service['nodetypes'] = array_filter($service['nodetypes']);

  $service['whentoping'] = $service['when_active'] * MULTIPING_WHEN_ACTIVE +
    $service['when_taxonomy'] * MULTIPING_WHEN_TAXONOMY +
    $service['when_nodetype'] * MULTIPING_WHEN_NODETYPE +
    $service['when_frontpage'] * MULTIPING_WHEN_FRONTPAGE;

  if ($service['id']) {
    drupal_write_record('multiping', $service, 'id');
  }
  else {
    drupal_write_record('multiping', $service);
    $form_state['redirect'] = 'admin/config/services/multiping';
  }

  drupal_set_message(t('Changes saved.'));
}

function multiping_delete_form($form, $form_state, $service) {
  $form['service'] = array(
    '#type' => 'value',
    '#value' => $service,
  );

  return confirm_form(
    $form,
    t('Are you sure you want to delete the ping service %service?', array('%service' => $service['name'])),
    'admin/config/services/multiping',
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

function multiping_delete_form_submit($form, &$form_state) {
  $service = $form_state['values']['service'];
  db_delete('multiping')->condition('id', $service['id'])->execute();
  drupal_set_message(t('Service %service has been deleted.', array('%service' => $service['name'])));
  watchdog('multiping', 'Service %service has been deleted.', array('%service' => $service['name']));
}
